/*****************************************************************************
* File Name: accelerometer.c
*
* Version 1.0
*
* Description:
*  This file provides functions to configure the onboard accelerometer of the 
*  PSoC 4 M-Series Pioneer Kit. This file also defines function to read the 
*  acceleration data from the accelerometer.
******************************************************************************
* Copyright (2015), Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*****************************************************************************/
#include "accelerometer.h"
#include "project.h"

/*****************************************************************************
* Local Function Prototypes
*****************************************************************************/
static uint8 Accelerometer_RegWrite(uint8 reg, uint8 value);


/*****************************************************************************
* Global Variable Declarations
*****************************************************************************/


/*****************************************************************************
* Local Global Variable Declarations
*****************************************************************************/


/*******************************************************************************
* Function Name: Accelerometer_RegWrite
********************************************************************************
* Summary:
* 	Write a single byte to a specified register location.
*
* Parameters:
*  uint8 reg: Register to which data is to be written.
*  uint8 *value: Variable which holds the data to be written.
*
* Return:
*  uint8: Status of write operation
*
*******************************************************************************/
static uint8 Accelerometer_RegWrite(uint8 reg, uint8 value)
{
	uint8 ackStatus = COMM_FAIL;
	ackStatus = I2C_I2CMasterSendStart(ACCELEROMETER_ADDR, I2C_I2C_WRITE_XFER_MODE);
	if(ackStatus == I2C_I2C_MSTR_NO_ERROR)
	{
		ackStatus = I2C_I2CMasterWriteByte(reg);
		ackStatus = I2C_I2CMasterWriteByte(value);
	}
	ackStatus = I2C_I2CMasterSendStop();
	return(ackStatus);
}

/*******************************************************************************
* Function Name: Accelerometer_RegRead
********************************************************************************
* Summary:
* 	Read a single byte from a specified register location.
*
* Parameters:
*  uint8 reg: Register from where data is to be read.
*  uint8 *value: Variable to which the data from register is stored.
*
* Return:
*  uint8: Status of read operation
*
*******************************************************************************/
uint8 Accelerometer_RegRead(uint8 reg, uint8 *value)
{
	uint8 ackStatus = COMM_FAIL;
    
	ackStatus = I2C_I2CMasterSendStart(ACCELEROMETER_ADDR, I2C_I2C_WRITE_XFER_MODE);
	if(ackStatus == I2C_I2C_MSTR_NO_ERROR)
	{
		ackStatus = I2C_I2CMasterWriteByte(reg);
		ackStatus = I2C_I2CMasterSendRestart(ACCELEROMETER_ADDR, I2C_I2C_READ_XFER_MODE);
		*value = I2C_I2CMasterReadByte(I2C_I2C_NAK_DATA);
	}
	ackStatus = I2C_I2CMasterSendStop();
	return(ackStatus);
}


/*******************************************************************************
* Function Name: ReadAccelerometer
********************************************************************************
* Summary:
* 	Reads the accelerometer acceleration values in x, y, and z directions.
*
* Parameters:
*  int16 *xData: Pointer to the variable in which the acceleration data 
*                for x-direction is to be stored. 
*
*  int16 *yData: Pointer to the variable in which the acceleration data 
*                for y-direction is to be stored. 
*
*  int16 *zData: Pointer to the variable in which the acceleration data 
*                for z-direction is to be stored. 
*
* Return:
*  void
*
*******************************************************************************/
uint8 SelfTest() {
    uint8 commStatus = COMM_FAIL;
    uint8 accRegVal;
    
    commStatus = Accelerometer_RegRead(DEVID, &accRegVal);
    if( commStatus == COMM_PASS ) {
        if ( accRegVal == 0xE5) {
            return COMM_PASS;
        }
        else {
            return COMM_FAIL;
        }
    }
    else {
        return COMM_FAIL;
    }
}
void ReadAccelerometer(uint16 *xData, uint16 *yData, uint16 *zData)
{
	/* Temporary variable for extracting only the 12 most significant
	 * bits of accelerometer data registers. */
	uint8 tempVal = ZERO;
	
	Accelerometer_RegRead(DATAX1, &tempVal);
	*xData = tempVal;
	Accelerometer_RegRead(DATAX0, &tempVal);
	*xData = (*xData << 8) | tempVal;
	if((*xData & 0x8000) == 0x8000)
	{
		*xData = *xData >> 4;
		*xData |= 0xF000;
	}
	else
	{
		*xData = *xData >> 4;
	}
	
	Accelerometer_RegRead(DATAY1, &tempVal);
	*yData = tempVal;
	Accelerometer_RegRead(DATAY0, &tempVal);
	*yData = (*yData << 8) | tempVal;
	
	if((*yData & 0x8000) == 0x8000)
	{
		*yData = *yData >> 4;
		*yData |= 0xF000;
	}
	else
	{
		*yData = *yData >> 4;
	}
	
	Accelerometer_RegRead(DATAZ1, &tempVal);
	*zData = tempVal;
	Accelerometer_RegRead(DATAZ0, &tempVal);
	*zData = (*zData << 8) | tempVal;
	
	if((*zData & 0x8000) == 0x8000)
	{
		*zData = *zData >> 4;
		*zData |= 0xF000;
	}
	else
	{
		*zData = *zData >> 4;
	}
}

void InitializeAccelerometer(void) {
    
    uint8 commStatus = COMM_FAIL;
    uint8 accRegValue = ZERO;
    uint8 accStatus = COMM_FAIL;
    
    accStatus = SelfTest();
    
    if(accStatus == COMM_PASS) {
    commStatus = Accelerometer_RegRead(POWER_CTL, &accRegValue);
    accRegValue = accRegValue & STANDBYMODE_AND;           // Accelerometer to Standby mode to update register values
    if (commStatus == COMM_PASS) {
        //UART_UartPutChar(0x10);
        commStatus = Accelerometer_RegWrite(POWER_CTL, accRegValue);
    }
    else {
        //UART_UartPutChar(0x01);
        commStatus = COMM_FAIL;
    }
    accRegValue = 0x01;
    if (commStatus == COMM_PASS)  {
        //UART_UartPutChar(0x20);
        commStatus = Accelerometer_RegWrite(THRESH_ACT, accRegValue);
    }
    else {
        //UART_UartPutChar(0x02);
        commStatus = COMM_FAIL;
    }
    accRegValue = 0x01;
    if (commStatus == COMM_PASS) {
        //UART_UartPutChar(0x30);
        commStatus = Accelerometer_RegWrite(THRESH_INACT, accRegValue);
    }
    else {
        //UART_UartPutChar(0x03);
        commStatus = COMM_FAIL;
    }
    accRegValue = 0x01;
    if (commStatus == COMM_PASS)  {
        //UART_UartPutChar(0x40);
        commStatus = Accelerometer_RegWrite(TIME_INACT, accRegValue);
    }
    else {
        //UART_UartPutChar(0x04);
        commStatus = COMM_FAIL;
    }
    
    
    if (commStatus == COMM_PASS)  {
        //UART_UartPutChar(0x50);
        commStatus = Accelerometer_RegRead(ACT_INACT_CTL, &accRegValue);
        
        /*  Activity - AC Operation
            Activity Enabled - x,y,z
            Inactivity -  DC Operation
            Inactivity Enabled - x,y,z */
        accRegValue = accRegValue | ACT_AC_OR | ACT_X_ENABLE_OR | ACT_Y_ENABLE_OR | ACT_Z_ENABLE_OR;
        accRegValue = accRegValue & INACT_DC_AND;
        accRegValue = accRegValue | INACT_X_ENABLE_OR | INACT_Y_ENABLE_OR | INACT_Z_ENABLE_OR;
        if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(ACT_INACT_CTL, accRegValue);
    }
    else {
        //UART_UartPutChar(0x05);
        commStatus = COMM_FAIL;
    }
    
    if (commStatus == COMM_PASS)  {
        //UART_UartPutChar(0x60);
        commStatus = Accelerometer_RegRead(BW_RATE, &accRegValue);
        
        /*  Operating Mode - NORMAL */
        accRegValue = accRegValue & NORMAL_AND;
        if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(BW_RATE, accRegValue);
    }
    else {
        //UART_UartPutChar(0x06);
        commStatus = COMM_FAIL;
    }
    
    
    if (commStatus == COMM_PASS)  {
        //UART_UartPutChar(0x70);
        commStatus = Accelerometer_RegRead(POWER_CTL, &accRegValue);
        
        /*  Linking of Delays - ON
            Auto Sleep - OFF
            Wake Up Device - ON*/
        accRegValue = accRegValue | LINK_OR;
        accRegValue = accRegValue & AUTO_SLEEP_OFF_AND & WAKEUP_AND;
        if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(POWER_CTL,accRegValue);
    }
    else {
        
        //UART_UartPutChar(0x07);
        commStatus = COMM_FAIL;
    }
    
    
    if (commStatus == COMM_PASS)  {
        //UART_UartPutChar(0x80);
        accRegValue = Accelerometer_RegRead(INT_ENABLE, &accRegValue);
        
        /*  Data Ready - OFF
            Single Tap - OFF
            Double Tap - OFF
            Activity - ON
            Inactivity - ON
            Free Fall - OFF
            Watermark - OFF
            Overrun - OFF */
        accRegValue = accRegValue & DATA_READY_DISABLE_AND & SINGLE_TAP_DISABLE_AND & DOUBLE_TAP_DISABLE_AND;
        accRegValue = accRegValue | ACTIVITY_ENABLE_OR | INACTIVITY_ENABLE_OR;
        accRegValue = accRegValue & FREE_FALL_DISABLE_AND & WATERMARK_DISABLE_AND & OVERRUN_DISABLE_AND;
        if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(INT_ENABLE, accRegValue);
    }
    else {
        
        //UART_UartPutChar(0x08);
        commStatus = COMM_FAIL;
    }
    
    
    if (commStatus == COMM_PASS) {
        //UART_UartPutChar(0x90);
        commStatus = Accelerometer_RegRead(INT_MAP, &accRegValue);
        
        /*  Activity - INT1
            Inactivity - INT1 */
        accRegValue = accRegValue & ACTIVITY_INT1_AND & INACTIVITY_INT1_AND;
        if (commStatus == COMM_PASS)  commStatus = Accelerometer_RegWrite(INT_MAP, accRegValue);
    }
    else {
        //UART_UartPutChar(0x09);
        commStatus = COMM_FAIL;
    }
    
    
    if (commStatus == COMM_PASS)  {
        //UART_UartPutChar(0xa0);
        commStatus = Accelerometer_RegRead(DATA_FORMAT, &accRegValue);
        
        /*  Self test - ON
            Full Resolution - 10bit
            Interrupts - Active Low
            Range - 2g (BIT1=0,BIT0=0) */
        accRegValue = accRegValue | SELF_TEST_APPLY_OR | FULL_RES_OR;
        accRegValue = accRegValue & INT_ACTIVE_LOW_AND & RANGE_BIT1_AND & RANGE_BIT0_AND;
        commStatus = Accelerometer_RegWrite(DATA_FORMAT, accRegValue);
    }
    else {
        //UART_UartPutChar(0x0a);
        commStatus = COMM_FAIL;
    }
    
    
    if (commStatus == COMM_PASS) {
        //UART_UartPutChar(0xb0);
        commStatus = Accelerometer_RegRead(POWER_CTL, &accRegValue);
        
        /* Back to Operation Mode */
        accRegValue = accRegValue | OUT_OF_STANDBY_OR;
        if (commStatus == COMM_PASS) Accelerometer_RegWrite(POWER_CTL, accRegValue);
    }
    else {
        //UART_UartPutChar(0x0b);
        commStatus = COMM_FAIL;
    }
    
    if (commStatus == COMM_PASS)  return;
    }
    else {
        //UART_UartPutChar(0xff);
        InitializeAccelerometer();
    }
}


void SleepAccelerometer(void) {
    
    uint8 commStatus = COMM_FAIL;
    uint8 accRegValue = ZERO;
    
    commStatus = Accelerometer_RegRead(POWER_CTL, &accRegValue);
    accRegValue = accRegValue & STANDBYMODE_AND;           // Accelerometer to Standby mode to update register values
    if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(POWER_CTL, accRegValue);
        /*  Updating Sleep value in the Power Control Register */
        accRegValue = accRegValue | GOTO_SLEEP_OR;
    if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(POWER_CTL, accRegValue);
        
}

void WakeAccelerometer(void) {
    uint8 commStatus = COMM_FAIL;
    uint8 accRegValue = ZERO;
    
    commStatus = Accelerometer_RegRead(POWER_CTL, &accRegValue);
    accRegValue = accRegValue & STANDBYMODE_AND;           // Accelerometer to Standy mode to update register values
    if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(POWER_CTL, accRegValue);
        /*  Updating Normal Operation value in the Power Control Register */
        accRegValue = accRegValue & WAKEUP_AND;
    if (commStatus == COMM_PASS) commStatus = Accelerometer_RegWrite(POWER_CTL, accRegValue);
}

/* [] END OF FILE */

